@@ -9,9 +9,9 @@
  ****************************************************************************** */
 /* eslint no-param-reassign: "error" */
 import _ from 'lodash';
-import KubeModel from './kube';
 import { responseHasError } from '../lib/utils';
 import logger from '../lib/logger';
+import GenericModel from './generic';
 
 export const ALL_SUBSCRIPTIONS = '__ALL__/SUBSCRIPTIONS__';
 const EVERYTHING_CHANNEL = '__ALL__/__ALL__//__ALL__/__ALL__';
@@ -22,7 +22,7 @@ export const filterByName = (names, items) => items.filter((item) => names.find(
 
 export const filterByNameNamespace = (names, items) => items.filter(({ metadata }) => names.find((name) => {
   const path = name.split('/');
-  return path[1] === metadata.name && path[0] === metadata.namespace;
+  return metadata && path.length === 2 && path[1] === metadata.name && path[0] === metadata.namespace;
 }));
 
 const longestCommonSubstring = (str1, str2) => {
@@ -115,8 +115,10 @@ export const getSubscriptionsDeployables = (allSubscriptions) => {
   // if a subscription has lots and lots of deployables, break into smaller subscriptions
   let allowAllChannel = true;
   const subscriptions = [];
+  let allDeployablePaths = 0;
   allSubscriptions.forEach((subscription) => {
     const deployablePaths = _.get(subscription, DEPLOYABLES, '').split(',').sort();
+    allDeployablePaths += deployablePaths.length;
 
     if (deployablePaths.length > 20) {
       const chunks = _.chunk(deployablePaths, 16);
@@ -129,11 +131,14 @@ export const getSubscriptionsDeployables = (allSubscriptions) => {
       chunks.forEach((chuck) => {
         subscriptions.push({ ...subscription, deployablePaths: chuck, isChucked: true });
       });
-      allowAllChannel = false;
     } else {
       subscriptions.push({ ...subscription, deployablePaths });
     }
   });
+  // hide all subscription option
+  if (allDeployablePaths > 100 || allSubscriptions.length <= 1) {
+    allowAllChannel = false;
+  }
 
   return { subscriptions, allowAllChannel };
 };
@@ -153,9 +158,17 @@ export const getAllChannels = (subscriptions, channels, selectedChannel, allowAl
   if (allowAllChannel) {
     if (channels.length > 1) {
       channels.unshift(EVERYTHING_CHANNEL);
+      // set default selectedSubscription when topology first render
+      if (!selectedSubscription) {
+        selectedSubscription = subscriptions.length > 0 ? [subscriptions[0]] : null;
+      }
     }
   } else if (!selectedSubscription) {
-    selectedSubscription = [subscriptions[0]];
+    selectedSubscription = subscriptions.length > 0 ? [subscriptions[0]] : null;
+  }
+  // renders all subscriptions when selected all subscriptions
+  if (allowAllChannel && selectedChannel === '__ALL__/__ALL__//__ALL__/__ALL__') {
+    selectedSubscription = subscriptions;
   }
   return selectedSubscription;
 };
@@ -251,7 +264,7 @@ export const buildDeployablesMap = (subscriptions, modelSubscriptions) => {
   };
 };
 
-export default class ApplicationModel extends KubeModel {
+export default class ApplicationModel extends GenericModel {
   // ///////////// CREATE APPLICATION ////////////////
   // ///////////// CREATE APPLICATION ////////////////
   // ///////////// CREATE APPLICATION ////////////////
@@ -282,6 +295,7 @@ export default class ApplicationModel extends KubeModel {
     const created = [];
     const updated = [];
     const errors = [];
+    const isEdit = !!deleteLinks;
 
     const checkAndCollectError = (response) => {
       if (response.code >= 400 || response.status === 'Failure' || response.message) {
@@ -318,12 +332,12 @@ export default class ApplicationModel extends KubeModel {
     // filter out Namespace resources that don't belong to App, Channel, Subscription
     // if a namespace belongs to app, etc, we create them all at once
     // if a namespace is independant, we create it separately (might be a secret, etc)
-    resources = resources.filter(({ kind, metadata = {} }) => kind !== 'Namespace' || !namespaces.has(metadata.name));
+    // ignore reqource path with no apiVersion, such as deleteLinks
+    resources = resources.filter(({ kind, metadata = {} }) => kind !== undefined && (kind !== 'Namespace' || !namespaces.has(metadata.name)));
     namespaces = Array.from(namespaces);
 
     // get resource end point for each resource
-    const k8sPaths = await this.kubeConnector.get('/');
-    const requestPaths = await Promise.all(resources.map(async (resource) => this.getResourceEndPoint(resource, k8sPaths)));
+    const requestPaths = await Promise.all(resources.map(async (resource) => this.getResourceEndPoint(resource)));
     if (requestPaths.length > 0) {
       const missingTypes = [];
       const missingEndPoints = [];
@@ -348,21 +362,17 @@ export default class ApplicationModel extends KubeModel {
       return { errors };
     }
 
-    // if isCreate, try to create all resouces EXCEPT Application
-    // we don't want to create Application until all the other resources successfully created/updated
-    // if update, update the application too
+    // create/modify the Application last
     let applicationResource;
     let applicationRequestPath;
-    if (!deleteLinks) {
-      resources = resources.filter((resource, index) => {
-        if (resource.kind === 'Application') {
-          applicationResource = resource;
-          ([applicationRequestPath] = requestPaths.splice(index, 1));
-          return false;
-        }
-        return true;
-      });
-    }
+    resources = resources.filter((resource, index) => {
+      if (resource.kind === 'Application') {
+        applicationResource = resource;
+        ([applicationRequestPath] = requestPaths.splice(index, 1));
+        return false;
+      }
+      return true;
+    });
 
     // if there's a namespace, try to create it
     if (namespaces.length === 0) {
@@ -372,7 +382,10 @@ export default class ApplicationModel extends KubeModel {
 
     const existingNamespaces = await this.kubeConnector.get('/api/v1/namespaces');
     const namespaceResponses = await Promise.all(namespaces.map((ns) => {
-      const namespaceExists = existingNamespaces.items.find((existingNamespace) => existingNamespace.metadata.name === ns);
+      let namespaceExists = false;
+      if (existingNamespaces && existingNamespaces.items) {
+        namespaceExists = existingNamespaces.items.find((existingNamespace) => existingNamespace.metadata.name === ns);
+      }
       return !namespaceExists ? this.createNamespace(ns) : undefined;
     }));
 
@@ -414,21 +427,22 @@ export default class ApplicationModel extends KubeModel {
 
     // if the only errors were "already existing", patch those resources
     if (errors.length === 0 && updates.length > 0) {
-      // get the selfLinks of the existing resources
-      const existing = await Promise.all(updates.map(({ requestPath, resource }) => {
+      // Update the existing resources
+      const replaced = await Promise.all(updates.map(({ requestPath, resource }) => {
         const name = _.get(resource, 'metadata.name');
-        return this.kubeConnector.get(`${requestPath}/${name}`);
-      }));
-
-      // then update the resources
-      const replaced = await Promise.all(updates.map(({ resource }, index) => {
-        const selfLink = _.get(existing, `[${index}].metadata.selfLink`);
-        const resourceVersion = _.get(existing, `[${index}].metadata.resourceVersion`);
-        _.set(resource, 'metadata.resourceVersion', resourceVersion);
-        const requestBody = {
-          body: resource,
-        };
-        return this.kubeConnector.put(`${selfLink}`, requestBody);
+        const path = `${requestPath}/${name}`;
+        return this.kubeConnector.get(path)
+          .then((existing) => {
+            const resourceVersion = _.get(existing, 'metadata.resourceVersion');
+            _.set(resource, 'metadata.resourceVersion', resourceVersion);
+            const requestBody = {
+              body: resource,
+            };
+            return this.kubeConnector.put(path, requestBody);
+          }).catch((err) => {
+            logger.error(err);
+            throw err;
+          });
       }));
 
       // report any errors
@@ -442,16 +456,19 @@ export default class ApplicationModel extends KubeModel {
 
     // if everything else created/updated
     if (errors.length === 0) {
-      // if update, delete any removed resources
-      if (deleteLinks) {
+      // if update
+      if (isEdit) {
+        // delete any removed resources except placement rules
         const destroyResponses = await Promise.all(Object.values(deleteLinks)[0]
-          .filter((link) => link.indexOf('/placementrules/') === -1)
-          .map((link) => this.kubeConnector.delete(link)))
+          .filter((link) => link.kind !== 'PlacementRule' && (!link.selfLink || link.selfLink.indexOf('/placementrules/') === -1))
+          .map((link) => this.deleteResource(link)))
           .catch((err) => ({
             status: 'Failure',
             message: err.message,
           }));
         checkAndCollectError(destroyResponses);
+
+        // don't update application--constantly changing
       } else {
         // if create, create the application
         const deployment = await this.kubeConnector.post(applicationRequestPath, applicationResource)
@@ -470,42 +487,17 @@ export default class ApplicationModel extends KubeModel {
     };
   }
 
-  async getResourceEndPoint(resource, k8sPaths) {
-    // dynamically get resource endpoint from kebernetes API
-    // ie.https://ec2-54-84-124-218.compute-1.amazonaws.com:8443/kubernetes/
-    if (k8sPaths) {
-      const { apiVersion, kind } = resource;
-      const apiPath = k8sPaths.paths.find((path) => path.match(`/[0-9a-zA-z]*/?${apiVersion}`));
-      if (apiPath) {
-        return (async () => {
-          const k8sResourceList = await this.kubeConnector.get(`${apiPath}`);
-          const resourceType = k8sResourceList.resources.find((item) => item.kind === kind);
-          const namespace = _.get(resource, 'metadata.namespace');
-          const { name, namespaced } = resourceType;
-          if (namespaced && !namespace) {
-            return null;
-          }
-          const requestPath = namespaced ? `namespaces/${namespace}/` : '';
-          return `${apiPath}/${requestPath}${name}`;
-        })();
-      }
-    }
-    return undefined;
-  }
-
   async createNamespace(namespace) {
-    const body = {
-      apiVersion: 'v1',
-      kind: 'Namespace',
-      metadata: {
-        name: namespace,
-      },
-    };
-    let response = await this.kubeConnector.post('/api/v1/namespaces', body);
+    let response = await this.kubeConnector.post('/apis/project.openshift.io/v1/projectrequests', { metadata: { name: namespace } }).catch((err) => {
+      logger.error(err);
+      throw err;
+    });
     if (responseHasError(response)) {
       if (response.code === 409) {
         response = await this.kubeConnector.get(`/api/v1/namespaces/${namespace}`);
       }
+    } else {
+      this.updateUserNamespaces(response);
     }
     return response;
   }
@@ -588,6 +580,9 @@ export default class ApplicationModel extends KubeModel {
         // what subscriptions does user want to see
         model.channels = [];
         model.subscriptions = [];
+        model.allSubscriptions = allSubscriptions;
+        model.allChannels = [];
+        model.allClusters = [];
 
         // get all the channels and find selected subscription from selected channel
         const subscr = getAllChannels(
@@ -604,7 +599,9 @@ export default class ApplicationModel extends KubeModel {
         await this.getAppDeployables(deployableMap, namespace, selectedSubscription, subscriptions);
         await this.getAppHooks(preHooksMap, true);
         await this.getAppHooks(postHooksMap, false);
-        await this.getAppRules(rulesMap);
+        await this.getAppRules(rulesMap, model.allClusters);
+        // get all channels
+        await this.getAllAppChannels(model.allChannels, allSubscriptions);
         if (includeChannels) {
           await this.getAppChannels(channelsMap);
         }
@@ -701,7 +698,38 @@ export default class ApplicationModel extends KubeModel {
     return Promise.all(requests);
   }
 
-  async getAppRules(rulesMap) {
+  // get all channels for all subscriptions
+  // this is used to build the subscription cards information
+  async getAllAppChannels(appAllChannels, allSubscriptions) {
+    let requests;
+    try {
+      // get all channels information
+      const channelsMap = {};
+      allSubscriptions.forEach((subscription) => {
+        const chnlData = _.get(subscription, 'spec.channel', '').split('/');
+        if (chnlData.length === 2) {
+          // eslint-disable-next-line prefer-destructuring
+          channelsMap[chnlData[0]] = chnlData[1];
+        }
+      });
+      requests = Object.entries(channelsMap).map(async ([channelNS, channelName]) => {
+        const response = await this.kubeConnector.getResources(
+          (ns) => `/apis/apps.open-cluster-management.io/v1/namespaces/${ns}/channels/${channelName}`,
+          { kind: 'Channel', namespaces: [channelNS] },
+        ) || [];
+        // stuff response into appAllChannels
+        response.forEach((channel) => {
+          appAllChannels.push(channel);
+        });
+      });
+    } catch (err) {
+      logger.error(err);
+      throw err;
+    }
+    return Promise.all(requests);
+  }
+
+  async getAppRules(rulesMap, allClusters) {
     let requests;
     try {
       requests = Object.entries(rulesMap).map(async ([namespace, values]) => {
@@ -717,6 +745,14 @@ export default class ApplicationModel extends KubeModel {
           values.forEach(({ ruleName, subscription }) => {
             if (name === ruleName) {
               subscription.rules.push(rule);
+              const clusters = _.get(rule, 'status.decisions', []);
+              clusters.forEach((cluster) => {
+                // get cluster name
+                const clusterName = _.get(cluster, 'clusterName');
+                if (clusterName && allClusters.indexOf(clusterName) === -1) {
+                  allClusters.push(clusterName);
+                }
+              });
             }
           });
         });
